/*
Gnome-shell extension for Liled
Copyright (C) 2012  Yannick Poirier

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */

const Soup = imports.gi.Soup;
const Main=imports.ui.main;
const GLib = imports.gi.GLib;
const Lang = imports.lang;

const Liled = new Lang.Class({

	Name: 'Liled.Liled',
	_init: function() {
		this._httpSession = new Soup.SessionAsync();
		Soup.Session.prototype.add_feature.call( this._httpSession, new Soup.ProxyResolverDefault());
		this._username = "";
		this._password = "";
		this._basic_auth = "Basic ";
		this._baseUrl = "";

		this.feeds = undefined;
		this.folders = undefined;
	},
	login: function( baseURL, username, password, callback ) {

		let endWith = "/app/index.php";
		if( baseURL.indexOf(endWith, baseURL.length - endWith.length) !== -1 )
			baseURL = baseURL.substring(0,  baseURL.length - endWith.length );
		this._baseUrl = baseURL + "/api/liled.php?format=json&action=";
		this._username = username;
		this._password = password;
		this._basic_auth = "Basic " + GLib.base64_encode( this._username + ":" + this._password );
		this.feeds = undefined;
		this.folders = undefined;

		let  request = Soup.Message.new('GET', this._baseUrl + "login");
		request.request_headers.append( 'Authorization', this._basic_auth );
		request.callback = callback;
		request.updateData = Lang.bind(this, this._setLogin);
		this._httpSession.queue_message(request, Lang.bind(this, this._httpCallback)  );
	},
	getFeeds: function( folderId ) {
		ret = new Array();
		if( this.feeds === undefined )
			return ret;

		for( let i = 0 ; i < this.feeds.length ; i++ ){
			feed = this.feeds[i];
			if( feed.folderID == folderId)
				ret.push(feed);
		}
		return ret;
	},
	updateFeeds: function(callback) {
		let  request = Soup.Message.new('GET', this._baseUrl + "feeds");
		request.request_headers.append( 'Authorization', this._basic_auth );
		request.callback = callback;
		request.updateData = Lang.bind(this, this._setFeeds);
		this.feeds = undefined;
		this._httpSession.queue_message(request, Lang.bind(this, this._httpCallback)  );
	},
	updateFolders: function(callback) {
		let  request = Soup.Message.new('GET', this._baseUrl + "folders");
		request.request_headers.append( 'Authorization', this._basic_auth );
		request.callback = callback;
		request.updateData = Lang.bind(this, this._setFolders);
		this.folders = undefined;
		this.feeds = undefined;
		this._httpSession.queue_message(request, Lang.bind(this, this._httpCallback)  );
	},
	_setFeeds: function( feeds ) {
		this.feeds = feeds;
	},
	_setFolders: function( folders ) {
		this.folders = folders; 
	},
	_setLogin: function( user ) {
		this.user = user;
	},
	_httpCallback: function( htpSession, message ) {
		if (message.status_code !== 200) {
			//message.callback( false, "Fail to get " + this._baseUrl );
			message.callback( false, null );
			return;
		}

		let response = message.response_body.data;
		let parsed = JSON.parse(response);
		if( !(parsed.error === undefined) ) {
			message.callback( false, parsed.reason );
			return;
		}

		message.updateData( parsed );
		message.callback( true, null );		
	}

});

